#version 330 core
layout(location = 0) out vec4 FragColor;
layout(location = 1) out vec4 BrightColor;

in	vec3 FragPos;
in  vec3 Normal;
in	vec2 TexCoords;
in	vec3 TangentLightPos;
in	vec3 TangentViewPos;
in	vec3 TangentFragPos;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture2;
uniform sampler2D normalMap;

uniform vec3 lightColor;
uniform vec3 lightPos;
uniform vec3 viewPos;
uniform bool norm;

void main()
{    
    if (norm) {
        // obtain normal from normal map in range [0,1]
        vec3 normal = texture(normalMap, TexCoords).rgb;
        // transform normal vector to range [-1,1]
        normal = normalize(normal * 2.0 - 1.0);  // this normal is in tangent space

        // get diffuse color
        vec4 objectColor = mix(texture(texture_diffuse1, TexCoords), texture(texture2, TexCoords), 0.1);
        // ambient
        float ambientStrength = 0.1;
        vec3 ambient = ambientStrength * lightColor;
        // diffuse
        vec3 lightDir = normalize(TangentLightPos - TangentFragPos);
        float diff = max(dot(lightDir, normal), 0.0);
        vec3 diffuse = diff * lightColor;
        // specular
        vec3 viewDir = normalize(TangentViewPos - TangentFragPos);
        vec3 reflectDir = reflect(-lightDir, normal);
        vec3 halfwayDir = normalize(lightDir + viewDir);
        float spec = pow(max(dot(normal, halfwayDir), 0.0), 32.0);

        vec3 specular = vec3(0.2) * spec;
        vec4 result = vec4(ambient + diffuse + specular, 1.0) * objectColor;
        FragColor = result;



    }
    else {

        vec4 objectColor = mix(texture(texture_diffuse1, TexCoords), texture(texture2, TexCoords), 0.1);
        //ambient
        float ambientStrength = 0.1;
        vec3 ambient = ambientStrength * lightColor;

        //diffuse
        vec3 norm = normalize(Normal);
        vec3 lightDir = normalize(lightPos - FragPos);

        float diff = max(dot(norm, lightDir), 0.0);
        vec3 diffuse = diff * lightColor;
        vec4 result = vec4(ambient + diffuse, 1.0) * objectColor;

        FragColor = vec4(result);
        // check whether result is higher than some threshold, if so, output as bloom threshold color
        float brightness = dot(FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
        if (brightness > 10.0)
            BrightColor = vec4(FragColor.rgb, 1.0);
        else
            BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
        FragColor = result;
    }
}