

#version 330 core
layout(location = 0) out vec4 FragColor;
layout(location = 1) out vec4 BrightColor;

in VS_OUT{
    vec3 FragPos;
    vec2 TexCoords;
    vec3 Normal;
    vec3 TangentLightPos;
    vec3 TangentViewPos;
    vec3 TangentFragPos;
} fs_in;

uniform int norm;
uniform sampler2D diffuseMap;
uniform sampler2D crystalNormal;

uniform vec3 lightPos;
uniform vec3 viewPos;


void main()
{

      

    if (norm == 1) {
        // obtain normal from normal map in range [0,1]
        vec3 normal = texture(crystalNormal, fs_in.TexCoords).rgb;
        // transform normal vector to range [-1,1]
        normal = normalize(normal * 2.0 - 1.0);  // this normal is in tangent space

    // get diffuse color
        vec3 color = texture(diffuseMap, fs_in.TexCoords).rgb;
        // ambient
        vec3 ambient = 0.1 * color;
        // diffuse
        vec3 lightDir = normalize(fs_in.TangentLightPos - fs_in.TangentFragPos);
        float diff = max(dot(lightDir, normal), 0.0);
        vec3 diffuse = diff * color;
        // specular
        vec3 viewDir = normalize(fs_in.TangentViewPos - fs_in.TangentFragPos);
        vec3 reflectDir = reflect(-lightDir, normal);
        vec3 halfwayDir = normalize(lightDir + viewDir);
        float spec = pow(max(dot(normal, halfwayDir), 0.0), 32.0);

        vec3 specular = vec3(0.2) * spec;
        vec4 result = vec4(ambient + diffuse + specular, 1.0);
        FragColor = vec4(result);

        //This codepart is only for testing the normals
        //FragColor = vec4(normal,1);
        
        // check whether result is higher than some threshold, if so, output as bloom threshold color
        float brightness = dot(FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
        if (brightness > 10.0)
            BrightColor = vec4(FragColor.rgb, 1.0);
        else
            BrightColor = vec4(0.0, 0.0, 0.0, 1.0);

    }
    else {
        vec3 normal = normalize(fs_in.Normal * 2.0 - 1.0);
        // get diffuse color
        vec3 color = texture(diffuseMap, fs_in.TexCoords).rgb;
        // ambient
        vec3 ambient = 0.1 * color;
        // diffuse
        vec3 lightDir = normalize(lightPos - fs_in.FragPos);
        float diff = max(dot(lightDir, normal), 0.0);
        vec3 diffuse = diff * color;
       

       
        vec4 result = vec4(ambient + diffuse, 1.0);

        FragColor = vec4(result);

        //This codepart is only for testing the normals
        //FragColor = vec4(normal,1);


        // check whether result is higher than some threshold, if so, output as bloom threshold color
        float brightness = dot(FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
        if (brightness > 10.0)
            BrightColor = vec4(FragColor.rgb, 1.0);
        else
            BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
    }
    }
    


